namespace Phoenix.Model.Bll.Domain
{
    using System;
    using System.Drawing;
    using System.IO;

    /// <summary>
    /// Represents a generic person in the domain
    /// </summary>
    public class Person : Poco
    {
        #region Fields

        private string sex = "F";

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Person"/> class.
        /// </summary>
        public Person()
        {
            this.LastUpdate = DateTime.Now;
            this.Counter = 0;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the address.
        /// </summary>
        /// <value>The address.</value>
        public virtual Address Address
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the value counting how many times this patient was used.
        /// </summary>
        /// <value>The counter.</value>
        public virtual long Counter
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the first name.
        /// </summary>
        /// <value>The first name.</value>
        public virtual string FirstName
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is complete. That's if the person was quickly
        /// inserted into the repository, he/she does not contains all the pieces of information and therefore
        /// need to be completed later.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is complete; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IsComplete
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether this patient is a male.
        /// </summary>
        /// <value><c>true</c> if the patient is a male; if the patient is a female, <c>false</c>.</value>
        public virtual bool IsMale
        {
            get
            {
                if (this.Sex.ToLower() == "m") return true;
                else return false;
            }
            set
            {
                this.Sex = (value) ? "M" : "F";
            }
        }

        /// <summary>
        /// Gets or sets the last name.
        /// </summary>
        /// <value>The last name.</value>
        public virtual string LastName
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the last update.
        /// </summary>
        /// <value>The last update.</value>
        public virtual DateTime LastUpdate
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the mail.
        /// </summary>
        /// <value>The mail.</value>
        public virtual string Mail
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the mobile pro.
        /// </summary>
        /// <value>The mobile pro.</value>
        public virtual string MobilePro
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the phone pro.
        /// </summary>
        /// <value>The phone pro.</value>
        public virtual string PhonePro
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sex.
        /// </summary>
        /// <value>The sex.</value>
        public virtual string Sex
        {
            get { return this.sex; }
            set
            {
                if (value.Length > 1
                    || (value.ToUpper() != "F" && value.ToUpper() != "M"))
                {
                    string msg = string.Format("The sex of a person should be 'F' or 'M' but it is '{0}'", value);
                    throw new ArgumentException(msg);
                }
                this.sex = value.ToUpper();
            }
        }

        /// <summary>
        /// Gets or sets the thumbnail.
        /// </summary>
        /// <value>The thumbnail.</value>
        public virtual byte[] Thumbnail
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the image.
        /// </summary>
        /// <value>The image.</value>
        public virtual Image ThumbnailImage
        {
            get
            {
                if (this.Thumbnail == null)
                {
                    return new Bitmap(1, 1);
                }
                else
                {
                    MemoryStream stream = new MemoryStream(this.Thumbnail);
                    return Image.FromStream(stream);
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("[{0}] {1,4} {2}", this.ID, this.FirstName, this.LastName);
        }

        #endregion Methods
    }
}