﻿namespace Phoenix.Model.Dal.Repositories
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Reflection;

    using NHibernate;
    using NHibernate.Linq;

    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal.Helpers;

    /// <summary>
    /// Provides the CRUD feature to whom implements this interface.
    /// </summary>
    /// <typeparam name="T">An item of the domain</typeparam>
    public abstract class Repository<T> : IRepository<T>
    {
        #region Fields

        private string orderField;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Repository&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="entityName">Name of the entity.</param>
        /// <param name="orderField">The field where the sort is done.</param>
        protected Repository(string entityName, string orderField)
        {
            this.EntityName = entityName;
            this.Session = OrmManager.Session;
            this.orderField = orderField;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the name of the entity.
        /// </summary>
        /// <value>The name of the entity.</value>
        protected string EntityName
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the session.
        /// </summary>
        /// <value>The session.</value>
        protected ISession Session
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the transaction.
        /// </summary>
        /// <value>The transaction.</value>
        protected ITransaction Transaction
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Creates the specified item in the database.
        /// </summary>
        /// <param name="item">The item.</param>
        public virtual void Create(T item)
        {
            this.Session.Save(item);
            this.Session.Flush();
        }

        /// <summary>
        /// Creates the specified items in the database and in one transaction.
        /// </summary>
        /// <param name="items">The items.</param>
        public void Create(IList<T> items)
        {
            this.BeginTransaction();
            foreach (T item in items)
            {
                this.Session.Save(item);
            }
            this.CommitTransaction();
        }

        /// <summary>
        /// Creates the or update the specified item agains the database.
        /// </summary>
        /// <param name="item">The item.</param>
        public void CreateOrUpdate(T item)
        {
            this.Session.SaveOrUpdate(item);
            this.Session.Flush();
        }

        /// <summary>
        /// Creates the or update the specified items agains the database and in one transaction.
        /// </summary>
        /// <param name="item">The item.</param>
        public void CreateOrUpdate(IList<T> items)
        {
            this.BeginTransaction();
            foreach (T item in items)
            {
                this.Session.SaveOrUpdate(item);
            }
            this.CommitTransaction();
        }

        /// <summary>
        /// Deletes the specified item from the database.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Delete(T item)
        {
            try
            {
                this.BeginTransaction();
                this.Session.Delete(item);
                //this.Session.Flush();
                this.CommitTransaction();
            }
            catch (Exception)
            {
                this.Session.CancelQuery();
                this.RollbackTransaction();
                throw;
            }
            finally
            {
                this.Session.Flush();
            }
        }

        /// <summary>
        /// Deletes the specified items from the database and in one transaction.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Delete(IList<T> items)
        {
            this.BeginTransaction();
            foreach (T item in items)
            {
                this.Session.Delete(item);
            }
            this.CommitTransaction();
            this.Session.Flush();
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            this.Session.Dispose();
            this.Transaction.Dispose();
        }

        /// <summary>
        /// Gets all the rows. Equivalent of a SELECT *
        /// </summary>
        /// <returns>All the rows of the repository</returns>
        public List<T> GetAllRows()
        {
            //string hql = string.Format("FROM {0} p", this.EntityName);
            //IQuery query = this.Session.CreateQuery(hql);
            //return (List<T>)query.List<T>();
            return (from item in this.Session.Linq<T>()
                    select item).ToList();
        }

        /// <summary>
        /// Gets all the rows. Equivalent of a SELECT *
        /// </summary>
        /// <param name="orderBy">The order by clause.</param>
        /// <returns>All the rows of the repository</returns>
        public List<T> GetAllRows(OrderBy orderBy)
        {
            string hql = string.Format("FROM {0} p ORDER BY {1} {2}"
                , this.EntityName, this.orderField, orderBy);

            IQuery query = this.Session.CreateQuery(hql);
            return (List<T>)query.List<T>();
        }

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        public abstract bool IsReferenced(T item);

        /// <summary>
        /// Orders the specified list of the patient.
        /// </summary>
        /// <param name="patient">The patient.</param>
        /// <param name="on">The list of item link to the patient this method has to order.</param>
        public void Order(IList<T> list, string propertyName)
        {
            list = list.OrderBy<T, object>(e =>
            {
                PropertyInfo info = e.GetType().GetProperty(propertyName);
                return info.GetValue(e, null);
            }).ToList();
        }

        /// <summary>
        /// Updates the specified item against the database.
        /// </summary>
        /// <param name="item">The item.</param>
        public virtual void Update(T item)
        {
            if (this.Session.Contains(item))
            {
                this.Session.Update(item);
                this.Session.Flush();
            }
        }

        /// <summary>
        /// Updates the specified items against the database and in one transaction.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Update(IList<T> items)
        {
            this.BeginTransaction();
            foreach (T item in items)
            {
                if (this.Session.Contains(item))
                    this.Session.Update(item);
            }
            this.CommitTransaction();
        }

        /// <summary>
        /// Begins the transaction.
        /// </summary>
        protected void BeginTransaction()
        {
            this.Transaction = this.Session.BeginTransaction();
        }

        /// <summary>
        /// Commits the transaction.
        /// </summary>
        protected void CommitTransaction()
        {
            this.Session.Flush();
            this.Transaction.Commit();
        }

        /// <summary>
        /// Rollbacks the transaction.
        /// </summary>
        protected void RollbackTransaction()
        {
            this.Transaction.Rollback();
        }

        #endregion Methods
    }
}