﻿namespace Phoenix.View.Frame
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Plugins.Core;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Presenter.Interfaces.Searcher;
    using Phoenix.Presenter.Sessions;
    using Phoenix.View.Configuration;
    using Phoenix.View.Helpers;
    using Phoenix.View.Input;
    using Phoenix.View.Management;

    using WeifenLuo.WinFormsUI.Docking;

    /// <summary>
    /// Main window and entry point of Phoenix Suite
    /// </summary>
    public partial class PhoenixMain : TranslateableForm, IPluginHost
    {
        #region Fields

        private const string SOFTWARE = "nDoctor 2";

        private readonly ListViewLogFrame listViewLogs = new ListViewLogFrame();
        private readonly SideMenu sideMenu;

        private AppearanceConfiguration appearanceConfig = new AppearanceConfiguration();
        private GeneralConfiguration generalConfig = new GeneralConfiguration();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Phoenix"/> class.
        /// </summary>
        public PhoenixMain()
            : base()
        {
            InitializeComponent();
            this.TranslateForm();
            this.Logger = LogManager.GetLogger(typeof(PhoenixMain));
            this.sideMenu = new SideMenu(this.dockPanel);

            AppContext.GlobalMessageSent += (sender, e) => this.ChangeStatus(e.Data);
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Adds a new menu.
        /// </summary>
        /// <param name="key">The key of the menu.</param>
        /// <param name="text">The text to dispay to the menu.</param>
        /// <param name="onClick">The process to execute when the menu is triggered.</param>
        /// <returns>
        /// 	<c>True</c> if added; otherwise <c>False</c>
        /// </returns>
        public bool AddMenu(string key, string text, Action<object, EventArgs> onClick)
        {
            if (AlreadyContains(key)) return false;

            var tsmi = new ToolStripMenuItem(text);
            tsmi.Name = key;
            tsmi.Click += (sender, e) => this.RaiseClick(sender, e, onClick);
            this.ms_MainMenu.Items.Add(tsmi);
            return true;
        }

        /// <summary>
        /// Adds a new menu.
        /// </summary>
        /// <param name="key">The key of the menu.</param>
        /// <param name="text">The text to dispay to the menu.</param>
        /// <returns>
        /// 	<c>True</c> if added; otherwise <c>False</c>
        /// </returns>
        public bool AddMenu(string key, string text)
        {
            return this.AddMenu(key, text, null);
        }

        /// <summary>
        /// Adds a separator to the menu with the specified key.
        /// </summary>
        /// <param name="menuName">Name of the menu.</param>
        /// <returns><c>True</c> if the sperator is added to the menu; otherwise <c>False</c></returns>
        public bool AddMenuSeparator(string menuName)
        {
            ToolStripMenuItem foundMenu = (from ToolStripMenuItem menu in this.ms_MainMenu.Items
                                           where menu.Name == menuName
                                           select menu).FirstOrDefault();

            if (foundMenu != null)
            {
                foundMenu.DropDownItems.Add(new ToolStripSeparator());
                return true;
            }
            else return false;
        }

        /// <summary>
        /// Adds a new side menu.
        /// </summary>
        /// <param name="key">The key of the SideMenu item.</param>
        /// <param name="text">The text displayed on the SideMenu item.</param>
        /// <param name="gui">The GUI item displayed in the SideMenu item to add.</param>
        /// <returns>
        /// 	<c>True</c> if added; otherwise <c>False</c>
        /// </returns>
        public bool AddSideMenu(string key, string text, object gui)
        {
            if (gui is Control)
            {
                this.sideMenu.AddMenu(key, text, gui as Control);
                return true;
            }
            else return false;
        }

        /// <summary>
        /// Adds a new submenu at the specified menu. If the menu doesn't exist, creates a new one
        /// </summary>
        /// <param name="menuName">The name of the menu .</param>
        /// <param name="submenuKey">The submenu key.</param>
        /// <param name="text">The text to display on the menu.</param>
        /// <param name="onClick">The process to execute when the menu is triggered.</param>
        /// <returns>
        /// 	<c>True</c> if added; otherwise <c>False</c>
        /// </returns>
        public bool AddSubmenu(Menus menuName, string submenuKey, string text, Action<object, EventArgs> onClick)
        {
            ToolStripMenuItem foundMenu = (from ToolStripMenuItem menu in this.ms_MainMenu.Items
                                           where menu.Name == GetName(menuName)
                                           select menu).FirstOrDefault();

            if (foundMenu == null) return false;
            if (AlreadyContains(foundMenu, submenuKey)) return false;

            var submenu = new ToolStripMenuItem(text);
            submenu.Name = submenuKey;
            submenu.Click += (sender, e) => this.RaiseClick(sender, e, onClick);
            foundMenu.DropDownItems.Add(submenu);
            return true;
        }

        /// <summary>
        /// Adds a new submenu at the specified menu. If the menu doesn't exist, nothing happen.
        /// </summary>
        /// <param name="menuName">The name of the menu .</param>
        /// <param name="submenuKey">The submenu key.</param>
        /// <param name="text">The text to display on the menu.</param>
        /// <param name="onClick">The process to execute when the menu is triggered.</param>
        /// <returns>
        /// 	<c>True</c> if added; otherwise <c>False</c>
        /// </returns>
        public bool AddSubmenu(string menuName, string submenuKey, string text, Action<object, EventArgs> onClick)
        {
            ToolStripMenuItem foundMenu = (from ToolStripMenuItem menu in this.ms_MainMenu.Items
                                           where menu.Name == menuName
                                           select menu).FirstOrDefault();

            if (foundMenu == null) return false;
            if (AlreadyContains(foundMenu, submenuKey)) return false;

            var submenu = new ToolStripMenuItem(text);
            submenu.Name = submenuKey;
            submenu.Click += (sender, e) => this.RaiseClick(sender, e, onClick);
            foundMenu.DropDownItems.Add(submenu);
            return true;
        }

        /// <summary>
        /// Loads the plugins.
        /// </summary>
        public void LoadPlugins()
        {
            try
            {
                var plugins = new PluginManager(AppContext.PluginRepositoryPath);
                plugins.LoadApplicationPlugins(this);
            }
            catch (Exception ex)
            {
                this.HandleError(ex);
            }
        }

        /// <summary>
        /// Removes the menu and all its submenus with the specified key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>
        /// 	<c>True</c> if removed; otherwise <c>False</c>
        /// </returns>
        public bool RemoveMenu(string key)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Removes the SideMenu item with the specified key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>
        /// 	<c>True</c> if removed; otherwise <c>False</c>
        /// </returns>
        public bool RemoveSideMenu(string key)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Removes the submenu with the specified key.
        /// </summary>
        /// <param name="menuKey">The menu key.</param>
        /// <param name="submenuKey">The submenu key.</param>
        /// <returns>
        /// 	<c>True</c> if removed; otherwise <c>False</c>
        /// </returns>
        public bool RemoveSubmenu(string menuKey, string submenuKey)
        {
            throw new NotImplementedException();
        }

        private bool AlreadyContains(string key)
        {
            var result = from ToolStripMenuItem menuItem in this.ms_MainMenu.Items
                         where menuItem.Name == key
                         select menuItem;
            return result.Count() > 0;
        }

        private bool AlreadyContains(ToolStripMenuItem menuItem, string key)
        {
            var result = from object subitem in menuItem.DropDownItems
                         where IsToolStripMenuItem(subitem)
                         && ((ToolStripMenuItem)subitem).Name == key
                         select subitem;
            return result.Count() > 0;
        }

        private void ChangeStatus(string message)
        {
            Assert.IsNotNull(message, "message");
            if (this.InvokeRequired) this.Invoke((Action)delegate { this.ChangeStatus(message); });
            else this.tssl_Status.Text = message;
        }

        private void DEBUG_Adddoctor(object sender, EventArgs e)
        {
            IDoctorInputPresenter fr = GuiManager.Get<IDoctorInputPresenter>();
            fr.Usage = Usage.Insert;

            GuiManager.ShowDialog(fr);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_AddDoctortype(object sender, EventArgs e)
        {
            IDoctorTypeInputPresenter dfr = GuiManager.Get<IDoctorTypeInputPresenter>();
            GuiManager.ShowDialog(dfr);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_AddDrug(object sender, EventArgs e)
        {
            IDrugInputPresenter box = GuiManager.Get<IDrugInputPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_AddDrugType(object sender, EventArgs e)
        {
            IDrugTypeInputPresenter box = GuiManager.Get<IDrugTypeInputPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_AddFamilyLink(object sender, EventArgs e)
        {
            IPatientSearcher search = GuiManager.Get<IPatientSearcher>();
            DialogResult dr = GuiManager.ShowDialog(search);
            if (dr != DialogResult.OK) return;

            IRelationInputPresenter box = GuiManager.Get<IRelationInputPresenter>();
            box.Patient = search.SelectedItem as Patient;

            GuiManager.ShowDialog(box);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_AddFamilyLinkType(object sender, EventArgs e)
        {
            IRelationTypeInputPresenter box = GuiManager.Get<IRelationTypeInputPresenter>();
            box.Usage = Usage.Insert;

            GuiManager.ShowDialog(box);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_AddInsurance(object sender, EventArgs e)
        {
            IInsuranceInputPresenter box = GuiManager.Get<IInsuranceInputPresenter>();
            GuiManager.ShowDialog(box);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_AddMacro(object sender, EventArgs e)
        {
            IMacroInputPresenter box = GuiManager.Get<IMacroInputPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_AddPathology(object sender, EventArgs e)
        {
            var box = GuiManager.Get<IPathologyInputPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_AddPathologyType(object sender, EventArgs e)
        {
            var box = GuiManager.Get<IPathologyTypeInputPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_AddPractice(object sender, EventArgs e)
        {
            IPracticeInputPresenter box = GuiManager.Get<IPracticeInputPresenter>();
            GuiManager.ShowDialog(box);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_AddReputation(object sender, EventArgs e)
        {
            IReputationInputPresenter box = GuiManager.Get<IReputationInputPresenter>();
            GuiManager.ShowDialog(box);
            this.sideMenu.RefreshGui();
        }

        private void DEBUG_Captcha(object sender, EventArgs e)
        {
            CaptchaBox fr = new CaptchaBox();
            fr.ConfirmationFailed += (x, y) => MessageBoxes.Asterisk("Captcha failed!");
            fr.ConfirmationSucceeded += (x, y) => MessageBoxes.Exclamation("Captcha succeeded!");
            fr.ShowDialog();
        }

        private void DEBUG_DoctorBox(object sender, EventArgs e)
        {
            DoctorInput box = new DoctorInput();
            box.ShowDialog();
        }

        private void DEBUG_ManageKeyword(object sender, EventArgs e)
        {
            IKeywordInputPresenter box = GuiManager.Get<IKeywordInputPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_ManageMedicalCardType(object sender, EventArgs e)
        {
            IMedicalCardTypeManagerPresenter box = GuiManager.Get<IMedicalCardTypeManagerPresenter>();
            GuiManager.ShowDialog(box);
        }

        private void DEBUG_OpenCalendar(object sender, EventArgs e)
        {
            var form = new CalendarBox();
            form.ShowDialog();
        }

        private void DEBUG_OpenMeetingBox(object sender, EventArgs e)
        {
            var form = new PatientMeetingInput();
            form.ShowDialog();
        }

        private void DEBUG_RefreshManagers(object sender, EventArgs e)
        {
            this.sideMenu.RefreshGui();
        }

        private void fr_ConfirmationFailed(object sender, EventArgs e)
        {
            this.NotYetImplemented();
        }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <param name="menuName">Name of the menu.</param>
        /// <returns></returns>
        private string GetName(Menus menuName)
        {
            switch (menuName)
            {
                case Menus.File: return "tsmi_File";
                case Menus.View: return "tsmi_View";
                case Menus.Session: return "tsmi_Session";
                case Menus.Tools: return "tsmi_Tools";
                case Menus.Help: return "tsmi_Help";
                default:
                    Assert.FailsOnEnumeration(menuName);
                    return "";
            }
        }

        private bool IsToolStripMenuItem(object subitem)
        {
            return subitem is ToolStripMenuItem;
        }

        private void MGMT_OpenCurrentDir(object sender, EventArgs e)
        {
            GuiManager.OpenCurrentDirectory();
        }

        private void MGMT_OpenLogDir(object sender, EventArgs e)
        {
            GuiManager.OpenLogDirectory();
        }

        private void NotYetImplemented()
        {
            MessageBoxes.Information("This feature is not yet implemented!");
        }

        private void PhoenixMain_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

        private void PhoenixMain_Load(object sender, EventArgs e)
        {
            try
            {
                //Login
                var login = new LoginForm();
                var dr = login.ShowDialog();
                if (dr != DialogResult.OK) Application.ExitThread();
                if (!login.Succeeded) Application.ExitThread();

                //Listen to new session:
                AppContext.SessionLoading += (sender2, e2) => this.sideMenu.Add(new Session(e2.Data));

                this.sideMenu.Hidden += sideMenu_Hidden;
                this.sideMenu.SearchPatientClicked += tsmi_OpenSession_Click;
                this.sideMenu.ShowHint = DockState.DockLeft;
                this.sideMenu.Show(this.dockPanel);
                this.Logger.Debug("SideMenu configured and loaded");

                this.listViewLogs.ShowHint = DockState.Document;
                this.listViewLogs.Show(this.dockPanel);
                this.Logger.Debug("LogFrame configured and loaded");

                this.tsmi_NavigationMenu.Checked = true;
            #if DEBUG
                this.tsmi_Debug.Visible = true;
                this.tsmi_LogFrame.Checked = true;
                this.listViewLogs.Show();

                using (new CursorSnapshot(Cursors.WaitCursor))
                {
                    DebugModel debug = new DebugModel();
                    if (debug.DebugPatient != null) this.sideMenu.Add(new Session(debug.DebugPatient));
                    else this.Logger.Warn("Couldn't find debug patient.");
                }
            #else
                this.tsmi_Debug.Visible = false;
                this.listViewLogs.Hide();
                this.WindowState = FormWindowState.Maximized;
            #endif
                this.RefreshShortcuts();
                this.Logger.Debug("Main frame loaded");
            }
            catch (Exception ex)
            {
                this.Logger.Error(ex);
                Application.Exit();
            }
        }

        private void RaiseClick(object sender, EventArgs e, Action<object, EventArgs> onClick)
        {
            if (onClick != null) onClick(sender, e);
        }

        private void RefreshShortcuts()
        {
            try
            {
                ShortcutConfiguration shortcuts = new ShortcutConfiguration();
                shortcuts.Load();

                this.tsmi_CloseSession.ShortcutKeys = shortcuts.Session_CloseSession;
                this.tsmi_OpenSession.ShortcutKeys = shortcuts.Session_OpenSession;
                this.tsmi_Configuration.ShortcutKeys = shortcuts.File_Configuration;
                this.tsmi_Quit.ShortcutKeys = shortcuts.File_Quit;
                this.tsmi_InsertPatient.ShortcutKeys = shortcuts.Session_InsertPatient;
                this.tsmi_NavigationMenu.ShortcutKeys = shortcuts.View_NavigationMenu;
                this.tsmi_LogFrame.ShortcutKeys = shortcuts.View_Log;
                this.tsmi_About.ShortcutKeys = shortcuts.Help_Help;
                this.Logger.Debug("Shortcuts refreshed.");
            }
            catch (Exception ex)
            {
                base.Logger.Error(ex);
                MessageBoxes.Error("Impossible to load the shortcuts: " + ex.Message);
            }
        }

        private void sideMenu_Hidden(object sender, EventArgs e)
        {
            this.tsmi_NavigationMenu.Checked = !this.tsmi_NavigationMenu.Checked;
        }

        private void tsmi_About_Click(object sender, EventArgs e)
        {
            AboutBox fr = new AboutBox();
            fr.ShowDialog();
        }

        private void tsmi_AllPictures_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<IPictureDisplayer>();
            GuiManager.Show(form, this.dockPanel);
        }

        private void tsmi_BugTracker_Click(object sender, EventArgs e)
        {
            BugTrackerForm form = new BugTrackerForm();
            form.Show();
        }

        private void tsmi_CalculateRelease_Click(object sender, EventArgs e)
        {
            ReleaseCalcBox box = new ReleaseCalcBox();
            GuiManager.ShowDialog(box);
        }

        private void tsmi_CloseSession_Click(object sender, EventArgs e)
        {
            ISessionDisposer box = GuiManager.Get<ISessionDisposer>();
            box.SetOpenedSessions(this.sideMenu.SessionsCollection);

            DialogResult dr = GuiManager.ShowDialog(box);

            if (dr == DialogResult.Cancel) return;

            var cursor = new CursorSnapshot();
            cursor.Set(Cursors.WaitCursor);

            foreach (var session in box.SessionsToClose) session.Close();

            this.sideMenu.RefreshSessions();

            cursor.Rollback();
        }

        private void tsmi_Configuration_Click(object sender, EventArgs e)
        {
            ConfigurationBox box = new ConfigurationBox();
            DialogResult dr = box.ShowDialog();

            if (dr == DialogResult.OK)
                this.RefreshShortcuts();
        }

        private void tsmi_DataAdministration_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<IAdministrationPresenter>();
            GuiManager.Show(form);
        }

        private void tsmi_DeletePatient_Click(object sender, EventArgs e)
        {
            IPatientSearcher fr = GuiManager.Get<IPatientSearcher>();
            DialogResult dr = GuiManager.ShowDialog(fr);
            if (dr == DialogResult.Cancel) return;

            var patient = fr.SelectedItem;
            if (this.sideMenu.Contains(fr.SelectedItem as Patient))
            {
                MessageBoxes.Exclamation("Before deleting this user, please close its session!");
            }
            else
            {
                var form = GuiManager.Get<IDeleter<Patient>>();
                form.ItemToDelete = patient as Patient;
                GuiManager.ShowDialog(form);
            }
        }

        private void tsmi_Import_Click(object sender, EventArgs e)
        {
            var s2pFile = Path.GetDirectoryName(Application.ExecutablePath);
            s2pFile = Path.Combine(s2pFile, "Phoenix.S2P.exe");
            if (!File.Exists(s2pFile))
            {
                MessageBoxes.Exclamation("The Sharpenda importer doesn't exist!");
                return;
            }

            var dr = MessageBoxes.Question(
                "In order to import data from an old Sharpenda database, nDoctor should close. Do you want to continue?");
            if (dr == DialogResult.No) return;

            Process.Start(s2pFile);
            this.Close();
        }

        private void tsmi_LogFrame_Click(object sender, EventArgs e)
        {
            if (this.listViewLogs == null)
            {
                this.Logger.Warn("Impossible to show the LogFrame because it is null!");
                return;
            }

            this.tsmi_LogFrame.Checked = !this.tsmi_LogFrame.Checked;
            if (tsmi_LogFrame.Checked)
                this.listViewLogs.Show();
            else
                this.listViewLogs.Hide();
        }

        private void tsmi_NavigationMenu_Click(object sender, EventArgs e)
        {
            if (tsmi_NavigationMenu.Checked)
            {
                this.sideMenu.Hide();
                this.tsmi_NavigationMenu.Checked = false;
            }
            else
            {
                this.sideMenu.Show();
                this.tsmi_NavigationMenu.Checked = true;
            }
        }

        private void tsmi_OpenSession_Click(object sender, EventArgs e)
        {
            IPatientSearcher fr = GuiManager.Get<IPatientSearcher>();
            DialogResult dr = GuiManager.ShowDialog(fr);
            if (dr == DialogResult.Cancel) return;

            AppContext.NotifySessionLoading(fr.SelectedItem as Patient);
        }

        private void tsmi_QuickInsertPatient_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<IPatientInserterPresenter>();
            GuiManager.Show(form);
        }

        private void tsmi_SaveAllAndClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void tsmi_UpdateUser_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<IUserInputPresenter>();
            form.Usage = Usage.Update;
            form.ItemToManage = AppContext.CurrentUser;
            GuiManager.ShowDialog(form);
        }

        #endregion Methods
    }
}