﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Windows.Forms;
    using System.Windows.Forms.Calendar;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Search;
    using Phoenix.Translation;
    using Phoenix.View.Input;

    using Colour = System.Drawing.Color;

    /// <summary>
    /// Manage the meetings
    /// </summary>
    public partial class CalendarBox : TranslateableForm, ICalendarPresenter
    {
        #region Fields

        private int dayToDisplay = 4;
        private CalendarModel model;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CalendarBox"/> class.
        /// </summary>
        public CalendarBox()
        {
            InitializeComponent();
            this.model = new CalendarModel(this);
            AppContext.WeekSelected += AppContext_WeekSelected;
            this.c_Calendar.TimeUnitsOffset = -16;

            this.tsmi_5min.Tag = CalendarTimeScale.FiveMinutes;
            this.tsmi_6min.Tag = CalendarTimeScale.SixMinutes;
            this.tsmi_10min.Tag = CalendarTimeScale.TenMinutes;
            this.tsmi_15min.Tag = CalendarTimeScale.FifteenMinutes;
            this.tsmi_30min.Tag = CalendarTimeScale.ThirtyMinutes;
            this.tsmi_60min.Tag = CalendarTimeScale.SixtyMinutes;

            this.tsmi_15min.Checked = true;
            this.c_Calendar.TimeScale = CalendarTimeScale.ThirtyMinutes;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user ask to display meetings in the specified range of time.
        /// </summary>
        public event EventHandler<EventArgs<DateTime, DateTime>> Displaying;

        /// <summary>
        /// Occurs when a meeting is deleted.
        /// </summary>
        public event EventHandler<EventArgs<Meeting>> MeetingDeleted;

        /// <summary>
        /// Occurs when a personal meeting is updated.
        /// </summary>
        public event EventHandler<EventArgs<Meeting>> MeetingUpdated;

        /// <summary>
        /// Occurs when meetings are updated.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> Updated;

        #endregion Events

        #region Methods

        /// <summary>
        /// Displays the specified meetings in the calendar.
        /// </summary>
        /// <param name="workMeetings">The work meetings.</param>
        /// <param name="meetings">The meetings.</param>
        public void Display(IList<WorkMeeting> workMeetings, IList<Meeting> meetings)
        {
            this.ClearListView();
            this.c_Calendar.Items.Clear();

            foreach (var each in workMeetings)
            {
                this.AddItemInCalendar(each);
                this.AddItemInListView(each);
            }

            foreach (var each in meetings)
            {
                this.AddItemInCalendar(each);
            }
        }

        private void AddItemInCalendar(WorkMeeting meeting)
        {
            var calItem = new CalendarItem(this.c_Calendar);
            calItem.StartDate = meeting.StartDate;
            calItem.EndDate = meeting.EndDate;
            calItem.Tag = meeting;
            calItem.Text = meeting.Title;
            calItem.BackgroundColor = Colour.Red;
            this.c_Calendar.Items.Add(calItem);
        }

        private void AddItemInCalendar(Meeting meeting)
        {
            var calItem = new CalendarItem(this.c_Calendar);
            calItem.StartDate = meeting.StartDate;
            calItem.EndDate = meeting.EndDate;
            calItem.Tag = meeting;
            calItem.Text = meeting.Title;
            calItem.BackgroundColor = Colour.Yellow;
            this.c_Calendar.Items.Add(calItem);
        }

        private void AddItemInListView(WorkMeeting meeting)
        {
            var query = (from ListViewGroup grp in this.lv_Agenda.Groups
                         where grp.Name == this.GetGroupName(meeting)
                         select grp).Count();

            if (query == 0) this.lv_Agenda.Groups.Add(this.GetGroupName(meeting), this.GetGroupName(meeting));

            ListViewItem item = new ListViewItem(this.GetText(meeting));
            item.Group = this.GetGroup(meeting);
            item.ImageKey = "event";
            this.lv_Agenda.Items.Add(item);
        }

        private void AppContext_WeekSelected(object sender, EventArgs<DateTime> e)
        {
            this.SetSelectedDays(e.Data);
        }

        private void CalendarBox_Load(object sender, EventArgs e)
        {
            this.DisplayThisWeek();
        }

        private void ClearListView()
        {
            this.lv_Agenda.Items.Clear();
            this.lv_Agenda.Groups.Clear();
        }

        private void CreateMeeting(DateTime start, DateTime end)
        {
            var patient = this.FindPatient();
            if (patient == null) return;

            var meetingBox = new PatientMeetingInput();
            meetingBox.Meeting = new WorkMeeting() { Patient = patient };
            meetingBox.SetTimeInterval(start, end);
            var dr = meetingBox.ShowDialog();
            if (dr != DialogResult.OK) return;

            patient.Add(meetingBox.Meeting);
            this.OnUpdated(patient);
        }

        private void CreatePersonalMeeting(DateTime start, DateTime end)
        {
            var meetingBox = new PersonalMeetingInput();
            meetingBox.Meeting = new Meeting();
            meetingBox.SetTimeInterval(start, end);
            var dr = meetingBox.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.OnMeetingUpdated(meetingBox.Meeting);
        }

        private void c_Calendar_ItemDatesChanged(object sender, CalendarItemEventArgs e)
        {
            var dr = this.UpdateMeeting(e.Item.Tag, e.Item.StartDate, e.Item.EndDate);
            this.SetSelectedDays(this.c_Calendar.ViewStart.GetMonday());
        }

        private void c_Calendar_ItemDeleting(object sender, CalendarItemCancelEventArgs e)
        {
            var dr = this.DeleteMeeting(e.Item.Tag);
            if (dr == DialogResult.No) e.Cancel = true;
        }

        private void c_Calendar_ItemDoubleClick(object sender, CalendarItemEventArgs e)
        {
            if (e.Item == null || e.Item.Tag == null) return;

            this.ShowMeeting(e.Item.Tag);
        }

        private DialogResult DeleteMeeting(object meeting)
        {
            if (meeting is WorkMeeting) return this.DeleteMeeting(meeting as WorkMeeting);
            else if (meeting is Meeting) return this.DeleteMeeting(meeting as Meeting);
            else
            {
                throw new ArgumentException(string.Format("Unexpected type {0}"
                    , meeting.GetType()), "meeting");
            }
        }

        private DialogResult DeleteMeeting(WorkMeeting meeting)
        {
            var patient = meeting.Patient;
            var dr = MessageBoxes.Question("Do you want to delete this event?".Translate());
            if (dr == DialogResult.No) return dr;

            patient.Remove(meeting);
            this.OnUpdated(patient);
            return dr;
        }

        private DialogResult DeleteMeeting(Meeting meeting)
        {
            var dr = MessageBoxes.Question("Do you want to delete this event?".Translate());
            if (dr == DialogResult.No) return dr;

            this.OnMeetingDeleted(meeting);
            return dr;
        }

        private void DisplayNextWeek()
        {
            var from = this.c_Calendar.ViewStart.GetMonday().AddDays(7);
            this.SetSelectedDays(from);
        }

        private void DisplayPreviousWeek()
        {
            var from = this.c_Calendar.ViewStart.GetMonday().AddDays(-7);
            this.SetSelectedDays(from);
        }

        private void DisplayThisWeek()
        {
            var monday = DateTime.Today.StartOfWeek(DayOfWeek.Monday);
            this.SetSelectedDays(monday);
        }

        private Patient FindPatient()
        {
            var patientBox = new PatientSearchBox();
            var dr = patientBox.ShowDialog();

            if (dr != DialogResult.OK) return null;
            else return patientBox.SelectedItem as Patient;
        }

        private ListViewGroup GetGroup(WorkMeeting meeting)
        {
            var query = (from ListViewGroup grp in this.lv_Agenda.Groups
                         where grp.Name == meeting.StartDate.ToShortDateString()
                         select grp).FirstOrDefault();
            if (query == null) throw new ArgumentException("The group doesn't exist for the current meeting");
            return query;
        }

        private string GetGroupName(WorkMeeting meeting)
        {
            return meeting.StartDate.Date.ToShortDateString();
        }

        private string GetText(WorkMeeting meeting)
        {
            return string.Format("[{0} - {1}] {2} {3}"
                , meeting.StartDate.ToShortTimeString()
                , meeting.EndDate.ToShortTimeString()
                , meeting.Patient.FirstName
                , meeting.Patient.LastName.ToUpper());
        }

        private void OnDisplaying(DateTime from, DateTime to)
        {
            this.Displaying.Raise(this, new EventArgs<DateTime, DateTime>(from, to));
        }

        private void OnMeetingDeleted(Meeting meeting)
        {
            this.MeetingDeleted.Raise(this, new EventArgs<Meeting>(meeting));
        }

        private void OnMeetingUpdated(Meeting meeting)
        {
            this.MeetingUpdated.Raise(this, new EventArgs<Meeting>(meeting));
            this.SetSelectedDays(this.c_Calendar.ViewStart.GetMonday());
        }

        private void OnUpdated(Patient patient)
        {
            this.Updated.Raise(this, new EventArgs<Patient>(patient));
            this.SetSelectedDays(this.c_Calendar.ViewStart.GetMonday());
            AppContext.NotifyMeetingUpdated();
        }

        private void SetSelectedDays(DateTime from)
        {
            DateTime to = from.AddDays(this.dayToDisplay);
            this.UpdateMonthLabel(from, to);
            this.c_Calendar.SetViewRange(from, to);
            this.OnDisplaying(from, to);
        }

        private void ShowMeeting(object meeting)
        {
            if (meeting is WorkMeeting) this.ShowMeeting(meeting as WorkMeeting);
            else if (meeting is Meeting) this.ShowMeeting(meeting as Meeting);
            else
            {
                throw new ArgumentException(string.Format("The type if {0} but Meeting or WorkMeeting was expectet"
                    , meeting.GetType()), "meeting");

            }
        }

        private void ShowMeeting(WorkMeeting meeting)
        {
            var form = new PatientMeetingInput();
            form.Meeting = meeting;
            var dr = form.ShowDialog();

            if (dr != DialogResult.OK) return;

            this.OnUpdated(form.Meeting.Patient);
        }

        private void ShowMeeting(Meeting meeting)
        {
            var form = new PersonalMeetingInput();
            form.Meeting = meeting;
            var dr = form.ShowDialog();

            if (dr != DialogResult.OK) return;

            this.OnMeetingUpdated(form.Meeting);
        }

        private void tsb_NextWeek_Click(object sender, EventArgs e)
        {
            this.DisplayNextWeek();
        }

        private void tsb_PreviousWeek_Click(object sender, EventArgs e)
        {
            this.DisplayPreviousWeek();
        }

        private void tsb_Today_Click(object sender, EventArgs e)
        {
            this.DisplayThisWeek();
        }

        private void tsd_NewMeeting_Click(object sender, EventArgs e)
        {
            this.tsd_NewMeeting.ShowDropDown();
        }

        private void tsmi_ContextAddPatientMeeting_Click(object sender, EventArgs e)
        {
            this.CreateMeeting(this.c_Calendar.SelectedElementStart.Date
                , this.c_Calendar.SelectedElementEnd.Date);
        }

        private void tsmi_ContextAddPersonalMeeting_Click(object sender, EventArgs e)
        {
            this.CreatePersonalMeeting(this.c_Calendar.SelectedElementStart.Date
                , this.c_Calendar.SelectedElementEnd.Date);
        }

        private void tsmi_PatientMeeting_Click(object sender, EventArgs e)
        {
            this.CreateMeeting(DateTime.Now, DateTime.Now.AddMinutes(30));
        }

        private void tsmi_PersonalMeeting_Click(object sender, EventArgs e)
        {
            this.CreatePersonalMeeting(DateTime.Now, DateTime.Now.AddMinutes(30));
        }

        private void tsmi_timescale_Click(object sender, EventArgs e)
        {
            this.c_Calendar.TimeUnitsOffset = DateTime.Now.Hour * (60 / (int)this.c_Calendar.TimeScale);
            this.tsmi_5min.Checked
                = this.tsmi_6min.Checked
                = this.tsmi_10min.Checked
                = this.tsmi_15min.Checked
                = this.tsmi_30min.Checked
                = this.tsmi_60min.Checked
                = false;

            var button = sender as ToolStripMenuItem;
            if (button != null)
            {
                button.Checked = true;
                this.c_Calendar.TimeScale = (CalendarTimeScale)button.Tag;
            }
        }

        private void tsmi_TimeSpan_ButtonClick(object sender, EventArgs e)
        {
            this.tsd_TimeSpan.ShowDropDown();
        }

        private void tsmi_Week_Click(object sender, EventArgs e)
        {
            this.tsmi_Week.Checked = true;
            this.tsmi_WorkWeek.Checked = false;
            this.dayToDisplay = 6;
            this.SetSelectedDays(this.c_Calendar.ViewStart);
        }

        private void tsmi_WorkWeek_Click(object sender, EventArgs e)
        {
            this.tsmi_WorkWeek.Checked = true;
            this.tsmi_Week.Checked = false;
            this.dayToDisplay = 4;
            this.SetSelectedDays(this.c_Calendar.ViewStart);
        }

        private DialogResult UpdateMeeting(object meeting, DateTime startDate, DateTime endDate)
        {
            if (meeting is WorkMeeting) return this.UpdateMeeting(meeting as WorkMeeting, startDate, endDate);
            else if (meeting is Meeting) return this.UpdateMeeting(meeting as Meeting, startDate, endDate);
            else
            {
                throw new ArgumentException(string.Format("Unexpected type {0}"
                    , meeting.GetType()), "meeting");
            }
        }

        private DialogResult UpdateMeeting(WorkMeeting meeting, DateTime startDate, DateTime endDate)
        {
            var dr = MessageBoxes.Question("Do you want to update this event?".Translate());
            if (dr == DialogResult.No) return dr;

            meeting.StartDate = startDate;
            meeting.EndDate = endDate;
            this.OnUpdated(meeting.Patient);
            return dr;
        }

        private DialogResult UpdateMeeting(Meeting meeting, DateTime startDate, DateTime endDate)
        {
            var dr = MessageBoxes.Question("Do you want to update this event?".Translate());
            if (dr == DialogResult.No) return dr;

            meeting.StartDate = startDate;
            meeting.EndDate = endDate;
            this.OnMeetingUpdated(meeting);
            return dr;
        }

        private void UpdateMonthLabel(DateTime from, DateTime to)
        {
            if (from.Month != to.Month && from.Year == to.Year)
            {
                this.tsl_SelectedWeek.Text = string.Format("{0} - {1}"
                    , (CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(from.Month) + " " + from.Year)
                    , (CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(to.Month) + " " + from.Year));
            }
            else
            {
                this.tsl_SelectedWeek.Text =
                    CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(from.Month)
                    + " "
                    + from.Year;
            }
        }

        #endregion Methods
    }
}