﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Globalization;
    using System.Linq;
    using System.Windows.Forms;
    using System.Windows.Forms.Calendar;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Gui item to manage the meetings of a patient
    /// </summary>
    public partial class MeetingManagerBox : TranslateableForm, IMeetingManagerPresenter
    {
        #region Fields

        private IList<WorkMeeting> meetings = new List<WorkMeeting>();
        private Patient patient;
        private MeetingModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MeetingManagerBox"/> class.
        /// </summary>
        public MeetingManagerBox()
        {
            InitializeComponent();
            this.presenter = new MeetingModel(this);
            this.c_Calendar.TimeUnitsOffset = -16;
        }

        #endregion Constructors

        #region Enumerations

        private enum SelectionMode
        {
            Week,
            WorkWeek
        }

        #endregion Enumerations

        #region Events

        /// <summary>
        /// Occurs when a meeting is deleted.
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when user wants to insert an item.
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when user wants to update an item.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Meetings".Translate(); }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.Meeting; }
        }

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public WorkMeeting ItemToManage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the list of all the meetings but these of the patient.
        /// </summary>
        /// <value>The meetings.</value>
        public IList<WorkMeeting> Meetings
        {
            get
            {
                return this.meetings;
            }
            set
            {
                this.meetings = value;
                this.RefreshGui();
            }
        }

        /// <summary>
        /// Gets or sets the patient.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get { return this.patient; }
            set
            {
                this.patient = value;
                this.RefreshGui();
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        public void RefreshGui()
        {
            #region Patient
            string title = "Meetings of {0}".Translate(this.Patient);
            this.Text = title;
            this.lbl_Patient.Text = title;
            #endregion

            #region Meetings
            this.tssl_Meetings.Text = "Meetings: {0}".Translate(this.Meetings.Count);
            this.SetCalendarInterval(this.c_Calendar.ViewStart, this.c_Calendar.ViewEnd);
            #endregion
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void AddItemInCalendar(WorkMeeting meeting, bool locked)
        {
            CalendarItem item = new CalendarItem(this.c_Calendar);
            item.StartDate = meeting.StartDate;
            item.EndDate = meeting.EndDate;
            item.Tag = meeting;

            if (locked)
            {
                item.Text = meeting.Title;
                item.Locked = true;
                item.BackgroundColor = Color.Red;
            }
            else
            {
                item.Text = this.Patient.ToString();
                item.Locked = false;
                item.BackgroundColor = Color.Blue;
            }

            this.c_Calendar.Items.Add(item);
        }

        private void cms_Calendar_Opening(object sender, System.ComponentModel.CancelEventArgs e)
        {
            this.tsmi_AddMeeting.Visible = this.ElementHasStartAndEnd();

            // Set to invisible to be sure, the delete menu won't be activated by accident
            this.tsmi_DeleteMeeting.Visible
                = this.tsmi_UpdateMeeting.Visible
                = false;
        }

        private void c_Calendar_ItemClick(object sender, CalendarItemEventArgs e)
        {
            this.ItemToManage = e.Item.Tag as WorkMeeting;

            this.tsmi_DeleteMeeting.Visible
                = this.tsmi_UpdateMeeting.Visible
                = this.ItemCanBeDeleted(e);
        }

        private void c_Calendar_ItemDatesChanged(object sender, CalendarItemEventArgs e)
        {
            DialogResult dr = MessageBoxes.Question("Do you want to update this meeting?".Translate());
            if (dr == DialogResult.No)
            {
                this.Refreshed.Raise(this);
                return;
            }

            this.ItemToManage = e.Item.Tag as WorkMeeting;
            this.ItemToManage.StartDate = e.Item.StartDate;
            this.ItemToManage.EndDate = e.Item.EndDate;

            this.Updated.Raise(this);
        }

        private void c_Calendar_ItemDoubleClick(object sender, CalendarItemEventArgs e)
        {
            WorkMeeting meeting = e.Item.Tag as WorkMeeting;
            new MeetingViewer(meeting).ShowDialog();
        }

        private void c_Calendar_MouseDown(object sender, MouseEventArgs e)
        {
            this.c_Calendar.AllowItemEdit = (e.Button == MouseButtons.Right)
                ? false
                : true;
        }

        private bool ElementHasStartAndEnd()
        {
            return this.c_Calendar.SelectedElementStart != null && this.c_Calendar.SelectedElementEnd != null;
        }

        private bool ItemCanBeDeleted(CalendarItemEventArgs e)
        {
            return !(e.Item.Locked || e.Item == null);
        }

        private void MeetingManagerBox_Load(object sender, EventArgs e)
        {
            this.SetCalendarInterval(DateTime.Today.GetMonday(), DateTime.Today.GetMonday().AddDays(4));
            this.Refreshed.Raise(this);

            this.tscb_SelectionMode.ComboBox.DataSource = Enum.GetValues(typeof(SelectionMode));
            this.tscb_SelectionMode.SelectedItem = SelectionMode.WorkWeek;
            this.mv_MonthView.SelectionMode = MonthView.MonthViewSelection.WorkWeek;
            this.mv_MonthView.Select(DateTime.Today.GetMonday(), DateTime.Today.GetMonday().AddDays(4));
        }

        private void mv_MonthView_SelectionChanged(object sender, EventArgs e)
        {
            DateTime from = mv_MonthView.SelectionStart;
            DateTime to = mv_MonthView.SelectionEnd;

            this.SetCalendarInterval(from, to);
            this.UpdateMonthLabel(from, to);
        }

        private void SetCalendarInterval(DateTime start, DateTime end)
        {
            this.c_Calendar.SetViewRange(start, end);

            //Display only in the interval
            this.c_Calendar.Items.Clear();
            var meetings = (from WorkMeeting m in this.Meetings
                            where m.StartDate <= end.AddDays(1)
                            && m.StartDate >= start.AddDays(-1)
                            select m).ToList();

            foreach (var meeting in meetings) this.AddItemInCalendar(meeting, true);
            foreach (var meeting in this.Patient.Meetings) this.AddItemInCalendar(meeting, false);
        }

        private void SetSelectedDays(DateTime from)
        {
            int days = ((SelectionMode)this.tscb_SelectionMode.SelectedItem == SelectionMode.Week)
                ? 6
                : 4;
            DateTime to = from.AddDays(days);
            this.SetCalendarInterval(from, to);
            this.UpdateMonthLabel(from, to);
            this.mv_MonthView.Select(from, to);
        }

        private void tsb_Next_Click(object sender, EventArgs e)
        {
            DateTime from = this.c_Calendar.ViewStart.GetMonday().AddDays(7);
            this.SetSelectedDays(from);
        }

        private void tsb_Previous_Click(object sender, EventArgs e)
        {
            DateTime from = this.c_Calendar.ViewStart.GetMonday().AddDays(-7);
            this.SetSelectedDays(from);
        }

        private void tsb_Today_Click(object sender, EventArgs e)
        {
            DateTime from = DateTime.Today.GetMonday();
            this.SetSelectedDays(from);
        }

        private void tscb_SelectionMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            SelectionMode mode = (SelectionMode)this.tscb_SelectionMode.ComboBox.SelectedItem;
            switch (mode)
            {
                case SelectionMode.Week:
                    this.mv_MonthView.SelectionMode = MonthView.MonthViewSelection.Week;
                    break;
                case SelectionMode.WorkWeek:
                    this.mv_MonthView.SelectionMode = MonthView.MonthViewSelection.WorkWeek;
                    break;
                default:
                    Assert.FailsOnEnumeration(mode);
                    break;
            }
            this.SetSelectedDays(this.c_Calendar.ViewStart);
        }

        private void tsmi_AddMeeting_Click(object sender, EventArgs e)
        {
            int minutes = (int)this.c_Calendar.TimeScale;
            DateTime from = this.c_Calendar.SelectedElementStart.Date;
            DateTime to = this.c_Calendar.SelectedElementEnd.Date.AddMinutes(minutes);

            if (from > to)
            {
                MessageBoxes.Warning("The end date of the meeting is before the start date! Try again with correct values.");
                return;
            }

            this.ItemToManage = new WorkMeeting()
            {
                StartDate = from,
                EndDate = to,
                Title = this.Patient.ToString(),
            };

            MeetingBox box = new MeetingBox();
            box.Meeting = this.ItemToManage;
            DialogResult dr = box.ShowDialog();

            if (dr == DialogResult.Cancel) return;

            this.c_Calendar.CreateItemOnSelection(this.Patient.ToString(), false);
            this.Inserted.Raise(this);
            this.tsmi_AddMeeting.Visible = true;
        }

        private void tsmi_DeleteMeeting_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBoxes.Question("Do you want to delete this meeting?".Translate());
            if (dr == DialogResult.No) return;

            this.Deleted.Raise(this);
            this.tsmi_DeleteMeeting.Visible = true;
        }

        private void tsmi_Update_Click(object sender, EventArgs e)
        {
            var box = new MeetingBox();
            box.Meeting = this.ItemToManage;
            DialogResult dr = box.ShowDialog();

            if (dr == DialogResult.Cancel) return;
            this.Updated.Raise(this);
        }

        private void UpdateMonthLabel(DateTime from, DateTime to)
        {
            if (from.Month != to.Month)
            {
                this.tsl_SelectedMonth.Text = string.Format("{0} - {1}",
                    (CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(from.Month) + " " + from.Year),
                    (CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(to.Month) + " " + from.Year));
            }
            else
            {
                this.tsl_SelectedMonth.Text =
                    CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(from.Month)
                    + " "
                    + from.Year;
            }
        }

        #endregion Methods
    }
}