﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.View.Input;

    /// <summary>
    /// Display and manage all the pathologic events of a patient.
    /// </summary>
    public partial class PathologiesHistoryManagerBox : TranslateableForm, IPathologiesHistoryPresenter
    {
        #region Fields

        private const string SERIE = "History";

        private PathologiesHistoryModel model;
        private SortBy sortBy = SortBy.StartDate;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PathologiesHistoryManagerBox"/> class.
        /// </summary>
        public PathologiesHistoryManagerBox()
        {
            InitializeComponent();
            this.model = new PathologiesHistoryModel(this);
            AppContext.DataChanged += AppContext_DataChanged;
        }

        #endregion Constructors

        #region Enumerations

        private enum SortBy
        {
            [Description("Start date")]
            StartDate,
            [Description("End date")]
            EndDate,
            [Description("Name")]
            Name,
            [Description("Type")]
            Type,
            [Description("Duration")]
            Duration,
        }

        #endregion Enumerations

        #region Events

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when the patient is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Pathologies"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.Pathology; }
        }

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Patient Patient
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data into the GUI.
        /// </summary>
        public void RefreshGui()
        {
            this.Refreshed.Raise(this);
            this.RefreshListView();
            this.RefreshChart();
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            AppContext.DataChanged -= AppContext_DataChanged;
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void AddPathology()
        {
            var form = new PathologicEventCreator();
            var dr = form.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.Patient.Add(form.ItemToManage);
            this.Updated.Raise(this);
            this.RefreshGui();
        }

        void AppContext_DataChanged(object sender, EventArgs e)
        {
            this.RefreshGui();
        }

        private void cms_Context_Opening(object sender, CancelEventArgs e)
        {
            this.tsmi_Delete.Visible
                = this.tsmi_Update.Visible
                = (this.lv_Pathologies.SelectedItems.Count != 0);
        }

        private void lv_Pathologies_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (this.lv_Pathologies.Columns[e.Column] == this.ch_StartDate) this.sortBy = SortBy.StartDate;
            else if (this.lv_Pathologies.Columns[e.Column] == this.ch_Type) this.sortBy = SortBy.Type;
            else if (this.lv_Pathologies.Columns[e.Column] == this.ch_EndDate) this.sortBy = SortBy.EndDate;
            else if (this.lv_Pathologies.Columns[e.Column] == this.ch_Pathology) this.sortBy = SortBy.Name;
            else if (this.lv_Pathologies.Columns[e.Column] == this.ch_TimeSpan) this.sortBy = SortBy.Duration;

            this.RefreshGui();
        }

        private void PathologiesHistoryManagerBox_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

        private void PathologiesHistoryManagerBox_Load(object sender, EventArgs e)
        {
            var x = this.Handle;

            if (this.DesignMode) return;
            else this.RefreshGui();
        }

        private void RefreshChart()
        {
            try
            {
                this.c_Chart.Series[SERIE].Points.Clear();

                int[] YAxes = new int[] { 0 };
                string[] XAxes = new string[] { "" };

                BackgroundWorker worker = new BackgroundWorker();
                worker.DoWork += (sender, e) =>
                {
                    YAxes = this.SortPathologies().Select(current => (current.EndDate - current.StartDate).Days).ToArray();
                    XAxes = this.SortPathologies().Select(current => current.Pathology.Title).ToArray();
                };
                worker.RunWorkerCompleted += (sender, e) => this.c_Chart.Series[SERIE].Points.DataBindXY(XAxes, YAxes);
                worker.RunWorkerAsync();
            }
            catch (Exception ex)
            {
                this.HandleError(ex);
                throw;
            }
        }

        private void RefreshListView()
        {
            this.lv_Pathologies.Items.Clear();

            foreach (var each in this.SortPathologies())
            {
                ListViewItem current = new ListViewItem();
                current.ImageKey = "event";
                current.Tag = each;
                current.SubItems.Add(each.Pathology.Type.Title);
                current.SubItems.Add(each.Pathology.Title);
                current.SubItems.Add(each.StartDate.ToShortDateString());
                current.SubItems.Add(each.EndDate.ToShortDateString());

                current.SubItems.Add(string.Format("{0} day(s)"
                    , (each.EndDate - each.StartDate).Days));

                this.lv_Pathologies.Items.Add(current);
            }
        }

        private List<PathologicEvent> SortPathologies()
        {
            switch (sortBy)
            {
                case SortBy.StartDate:
                    return this.Patient.Pathologies.OrderBy(e => e.StartDate).ToList();
                case SortBy.EndDate:
                    return this.Patient.Pathologies.OrderBy(e => e.EndDate).ToList();
                case SortBy.Name:
                    return this.Patient.Pathologies.OrderBy(e => e.Pathology.Title).ToList();
                case SortBy.Type:
                    return this.Patient.Pathologies.OrderBy(e => e.Pathology.Type.Title).ToList();
                case SortBy.Duration:
                    return this.Patient.Pathologies.OrderBy(e => (e.EndDate - e.StartDate).Days).ToList();
                default:
                    Assert.FailsOnEnumeration(sortBy);
                    return null;
            }
        }

        private void tsb_Add_Click(object sender, EventArgs e)
        {
            this.AddPathology();
        }

        private void tsb_Filter_Click(object sender, EventArgs e)
        {
            var filters = Enum.GetValues(typeof(SortBy));

            var form = new ComboBoxInput(sortBy);
            var dr = form.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.sortBy = (SortBy)form.SelectedItem;
            this.RefreshGui();
        }

        private void tsmi_Add_Click(object sender, EventArgs e)
        {
            this.AddPathology();
        }

        private void tsmi_Delete_Click(object sender, EventArgs e)
        {
            if (this.lv_Pathologies.SelectedItems.Count == 0) return;

            var current = this.lv_Pathologies.SelectedItems[0].Tag as PathologicEvent;
            this.Patient.Remove(current);
            this.Updated.Raise(this);
            this.RefreshGui();
        }

        private void tsmi_Update_Click(object sender, EventArgs e)
        {
            if (this.lv_Pathologies.SelectedItems.Count == 0) return;

            var form = new PathologicEventCreator(this.lv_Pathologies.SelectedItems[0].Tag as PathologicEvent);
            var dr = form.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.Update(form.ItemToManage);
            this.RefreshGui();
            this.Updated.Raise(this);
        }

        private void tssb_Refresh_Click(object sender, EventArgs e)
        {
            this.RefreshGui();
        }

        private void Update(PathologicEvent newValue)
        {
            var selected = this.lv_Pathologies.SelectedItems[0].Tag as PathologicEvent;
            selected.EndDate = newValue.EndDate;
            selected.StartDate = newValue.StartDate;
            selected.Remark = newValue.Remark;
            selected.Pathology = newValue.Pathology;
        }

        #endregion Methods
    }
}