﻿namespace Phoenix.View.Search
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Search;
    using Phoenix.Presenter.Interfaces.Searcher;
    using Phoenix.Presenter.Search;
    using Phoenix.Presenter.Search.Criterion;
    using Phoenix.Translation;

    /// <summary>
    /// Search box to find drugs
    /// </summary>
    public class DrugSearchBox : SearchBox, IDrugSearchBox
    {
        #region Fields

        private const string TITLE = "Name";
        private const string TYPE = "Type";

        private DrugSearcher model;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DrugSearchBox"/> class.
        /// </summary>
        public DrugSearchBox()
        {
            this.Text = "Search drugs";
            this.Logger = LogManager.GetLogger(typeof(DrugSearchBox));
            this.DefaultView = SearchBoxView.Tile;

            #region Columns

            base.AddListViewColumn("c_Type", "Type");
            base.AddListViewColumn("c_Name", "Name");

            #endregion

            #region Criterions
            int tabindex = 0;
            this.model = new DrugSearcher(this);

            TextCriterion name = new TextCriterion();
            name.CriterionName = TITLE;
            name.Text = "Name".Translate();
            name.IsActivated = true;
            name.TabIndex = tabindex++;
            name.Dock = DockStyle.Fill;
            this.Criterions.Add(name);

            ComboBoxCriterion type = new ComboBoxCriterion();
            type.CriterionName = TYPE;
            type.CriterionText = "Type".Translate();
            type.IsActivated = false;
            type.TabIndex = tabindex++;
            type.Dock = DockStyle.Fill;
            this.Criterions.Add(type);

            this.SetSplitterDistance();
            #endregion
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when an item is selected on the GUI.
        /// </summary>
        public event EventHandler<EventArgs<Drug>> ItemSelected;

        /// <summary>
        /// Occurs when searcher is loading.
        /// </summary>
        public event EventHandler Loading;

        /// <summary>
        /// Occurs when the user fire a search in the GUI
        /// </summary>
        public event EventHandler<EventArgs<Drug>> Searching;

        #endregion Events

        #region Methods

        /// <summary>
        /// Fills the specified items into the GUI.
        /// </summary>
        /// <param name="items">The items to load into the SearchBox.</param>
        public void Fill(IList<Drug> items)
        {
            try
            {
                this.ClearItemsFromListView();
                List<ListViewItem> lvitems = new List<ListViewItem>();
                foreach (var item in items)
                {
                    ListViewItem lvitem = new ListViewItem(item.Title);

                    ListViewItem.ListViewSubItem siType
                        = new ListViewItem.ListViewSubItem(lvitem, item.Type.Title);
                    this.SetImage(ItemType.Drug, lvitem);
                    lvitem.SubItems.Add(siType);
                    lvitem.Tag = item;
                    lvitems.Add(lvitem);
                }
                this.AddListViewItems(lvitems);
                this.AutoResizeListView();
                this.DeactivateProgressBar();
            }
            catch (Exception ex)
            {
                this.Logger.Warn(ex);
                MessageBoxes.Warning("An error occured while filling the result: " + ex.Message);
            }
        }

        /// <summary>
        /// Sets the drug types.
        /// </summary>
        /// <param name="drugTypes">The drug types.</param>
        public void SetDrugTypes(DrugType[] drugTypes)
        {
            var criterion = this.Criterions[TYPE] as ComboBoxCriterion;
            criterion.DataSource = drugTypes;
            criterion.DisplayMember = "Title";
        }

        /// <summary>
        /// Process to execute just after the parent form has executed the Load event
        /// </summary>
        protected override void LoadProcess()
        {
            this.OnLoading();
            base.LoadProcess();
        }

        /// <summary>
        /// Called when a Patient search is fired.
        /// The event args if a default instance because it is not used
        /// </summary>
        protected override void OnSearch()
        {
            try
            {
                this.ActivateProgressBar();

                this.Criterion = this.GetCriterion();
                this.Searching.Raise(this, new EventArgs<Drug>(new Drug()));
            }
            catch (Exception ex)
            {
                this.LogAndDisplayOnSearchError(ex);
            }
        }

        /// <summary>
        /// Called when a item selection is fired.
        /// </summary>
        protected override void OnSelection()
        {
            try
            {
                this.ItemSelected.Raise(this, new EventArgs<Drug>(this.SelectedItem as Drug));
            }
            catch (Exception ex)
            {
                this.LogAndDisplayOnSelectError(ex);
            }
        }

        private Drug GetCriterion()
        {
            var drug = new Drug();
            if (this.Criterions[TITLE].IsActivated)
            {
                drug.Title = (this.Criterions[TITLE].Value as string);
            }
            if (this.Criterions[TYPE].IsActivated)
            {
                drug.Type = this.Criterions[TYPE].Value as DrugType;
            }
            return drug;
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            //
            // DrugSearchBox
            //
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(496, 593);
            this.Name = "DrugSearchBox";
            this.ResumeLayout(false);
            this.PerformLayout();
        }

        private void OnLoading()
        {
            Loading.Raise(this);
        }

        #endregion Methods
    }
}