﻿namespace Phoenix.View.Search
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Search;
    using Phoenix.Presenter.Interfaces.Searcher;
    using Phoenix.Presenter.Search;
    using Phoenix.Presenter.Search.Criterion;
    using Phoenix.Translation;

    /// <summary>
    /// Gui item to search prescriptions of a patint.
    /// </summary>
    public class PrescriptionSearchBox : SearchBox, IPrescriptionSearcher
    {
        #region Fields

        private PrescriptionSearcher searcher;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PrescriptionSearchBox"/> class.
        /// </summary>
        public PrescriptionSearchBox()
        {
            this.Text = "Search prescriptions";
            this.Logger = LogManager.GetLogger(typeof(PrescriptionSearchBox));
            this.searcher = new PrescriptionSearcher(this);
            this.DefaultView = SearchBoxView.Tile;
            this.Patient = new Patient();

            #region Columns
            this.AddListViewColumn("c_Date", "Date");
            #endregion

            #region Criterions
            int tabindex = 0;
            DateTimeCriterion from = new DateTimeCriterion();
            from.Locked = true;
            from.CriterionName = "From";
            from.Text = "From".Translate();
            from.IsActivated = true;
            from.TabIndex = tabindex++;
            this.Criterions.Add(from);

            DateTimeCriterion to = new DateTimeCriterion();
            to.Locked = true;
            to.CriterionName = "To";
            to.Text = "To".Translate();
            to.IsActivated = true;
            from.TabIndex = tabindex++;
            this.Criterions.Add(to);

            this.SetSplitterDistance();
            #endregion
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when an item is selected on the GUI.
        /// </summary>
        public event EventHandler<EventArgs<Prescription>> ItemSelected;

        /// <summary>
        /// Occurs when the user fire a search in the GUI
        /// </summary>
        public event EventHandler<EventArgs<Prescription>> Searching;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the patient involved in the search.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Fills the specified items into the GUI.
        /// </summary>
        /// <param name="items">The items to load into the SearchBox.</param>
        public void Fill(IList<Prescription> items)
        {
            try
            {
                this.ClearItemsFromListView();
                List<ListViewItem> lvitems = new List<ListViewItem>();
                foreach (var item in items)
                {
                    string date = (item.CreationDate.HasValue)
                        ? item.CreationDate.Value.ToShortDateString()
                        : "N.A.";

                    ListViewItem lvitem = new ListViewItem(date);

                    lvitem.Tag = item;
                    this.SetImage(ItemType.Drug, lvitem);

                    lvitems.Add(lvitem);
                }
                this.AddListViewItems(lvitems);
                this.AutoResizeListView();
                this.DeactivateProgressBar();
            }
            catch (Exception ex)
            {
                this.Logger.Warn(ex);
                MessageBoxes.Warning("An error occured while filling the result: " + ex.Message);
            }
        }

        /// <summary>
        /// Called when a Patient search is fired.
        /// </summary>
        protected override void OnSearch()
        {
            Assert.IsTrue((this.Patient != null && !this.Patient.IsEmpty)
                , "Trying to search prescription of a patient while he's not set!");
            try
            {
                this.ActivateProgressBar();

                var criterion = new PrescriptionSearcher.Criterion();
                criterion.FromDate = (DateTime)this.Criterions["From"].Value;
                criterion.ToDate = (DateTime)this.Criterions["To"].Value;
                criterion.Patient = this.Patient;
                this.Criterion = criterion;

                this.Searching.Raise(this, new EventArgs<Prescription>(new Prescription()));
            }
            catch (Exception ex)
            {
                this.LogAndDisplayOnSearchError(ex);
            }
        }

        /// <summary>
        /// Called when a item selection is fired.
        /// </summary>
        protected override void OnSelection()
        {
            try
            {
                this.ItemSelected.Raise(this, new EventArgs<Prescription>(this.SelectedItem as Prescription));
            }
            catch (Exception ex)
            {
                this.LogAndDisplayOnSelectError(ex);
            }
        }

        #endregion Methods
    }
}