﻿namespace Phoenix.Presenter
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Helpers.Misc;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Sessions;
    using Phoenix.View;

    /// <summary>
    /// Provides features to close and save opened openedSessions.
    /// </summary>
    public partial class SessionDisposerBox : TranslateableForm, ISessionDisposer
    {
        #region Fields

        private readonly SessionCollection openedSessions = new SessionCollection();

        private SessionDisposer presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SessionDisposerBox"/> class.
        /// </summary>
        public SessionDisposerBox()
        {
            InitializeComponent();
            this.Logger = LogManager.GetLogger(typeof(SessionDisposerBox));
            this.presenter = new SessionDisposer(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user sessions are closing.
        /// </summary>
        public event EventHandler SessionsClosing;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets all the opened sessions.
        /// </summary>
        /// <value>The opened sessions.</value>
        public SessionCollection OpenedSessions
        {
            get
            {
                return this.openedSessions;
            }
        }

        /// <summary>
        /// Gets the patients who were in the closed sessions.
        /// </summary>
        /// <value>The closed patients.</value>
        public List<Session> SessionsToClose
        {
            get
            {
                SessionCollection sessions = new SessionCollection();
                foreach (object item in this.clb_Sessions.CheckedItems)
                {
                    Session session = item as Session;
                    if (session != null) sessions.Add(session);
                }
                return sessions;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Indicates the user the application has finished working.
        /// Typically hide the sandglass cursor show the default cursor
        /// </summary>
        public void EndGuiWait()
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.EndGuiWait(); });
            }
            else
            {
                this.Cursor = Cursors.Default;
            }
        }

        /// <summary>
        /// Add a list of opened sessions.
        /// </summary>
        /// <param name="sessions">The sessions.</param>
        public void SetOpenedSessions(SessionCollection sessions)
        {
            this.clb_Sessions.Items.Clear();
            this.openedSessions.Clear();
            this.openedSessions.AddRange(sessions);

            foreach (Session session in openedSessions)
            {
                this.clb_Sessions.Items.Add(session);
            }

            if (openedSessions.Count == 0)
            {
                this.btn_Close.Enabled = false;
                this.btn_CloseAll.Enabled = false;
            }
        }

        /// <summary>
        /// Indicates the user the application is working.
        /// Typically show the sandglass cursor
        /// </summary>
        public void StartGuiWait()
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.StartGuiWait(); });
            }
            else
            {
                this.Cursor = Cursors.WaitCursor;
            }
        }

        private void btn_Close_Click(object sender, EventArgs e)
        {
            if (sender == this.btn_CloseAll)
            {
                for (int i = 0; i < this.clb_Sessions.Items.Count; i++)
                {
                    this.clb_Sessions.SetItemChecked(i, true);
                }
            }

            this.SessionsClosing.Raise(this);
            base.DialogResult = DialogResult.OK;
            this.Close();
        }

        #endregion Methods
    }
}