﻿namespace Phoenix.Helpers.Misc
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;

    /// <summary>
    /// This report stores information about execution 
    /// time of an instance of a Benchmark.
    /// </summary>
    public interface IReport
    {
        #region Events

        /// <summary>
        /// Occurs when this report is updated.
        /// </summary>
        event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the name of the calling method.
        /// </summary>
        /// <value>The calling method.</value>
        string CallingMethod
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the elapsed time for the last execution.
        /// </summary>
        /// <value>The elapsed timed.</value>
        TimeSpan Elapsed
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the elapsed time in second for the last
        /// execution.
        /// </summary>
        /// <value>The elapsed time in seconds.</value>
        double ElapsedSeconds
        {
            get;
        }

        /// <summary>
        /// Gets or sets the start date and time.
        /// </summary>
        /// <value>The start.</value>
        DateTime Start
        {
            get;
        }

        /// <summary>
        /// Gets the start date and time as a string.
        /// </summary>
        /// <value>The start as string.</value>
        string StartAsString
        {
            get;
        }

        /// <summary>
        /// Gets or sets the total time of execution.
        /// </summary>
        /// <value>The total.</value>
        TimeSpan Total
        {
            get;
        }

        /// <summary>
        /// Gets the total time of execution in seconds.
        /// </summary>
        /// <value>The total seconds.</value>
        double TotalSeconds
        {
            get;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        object Clone();

        /// <summary>
        /// Return an object that contains all the
        /// history of the uses of the report specified in the
        /// constructor of this instance.
        /// </summary>
        /// <returns></returns>
        History Historise();

        #endregion Methods
    }

    /// <summary>
    /// This object calculate the elapsed time between 
    /// its instanciation and is destruction.
    /// </summary>
    public class Benchmark : IDisposable
    {
        #region Fields

        private string callingMethod = string.Empty;
        private List<IReport> reports = new List<IReport>();
        private Stopwatch stopwatch = new Stopwatch();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Benchmark"/> class.
        /// </summary>
        /// <param name="report">The report.</param>
        public Benchmark(IReport report)
        {
            StackFrame frame = new StackFrame(1);
            this.callingMethod = frame.GetMethod().DeclaringType + "." + frame.GetMethod().Name;

            stopwatch.Start();

            report.CallingMethod = this.callingMethod;
            this.reports.Add(report);
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Gets an instance of a report.
        /// </summary>
        /// <returns></returns>
        public static IReport GetReport()
        {
            return new Report();
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            TimeSpan elapsed = stopwatch.Elapsed;
            stopwatch.Stop();

            foreach (var report in this.reports)
            {
                report.Elapsed = elapsed;
            }
        }

        #endregion Methods
    }

    /// <summary>
    /// Represent a list of reports which represents
    /// a historisation of use of Benchmark
    /// </summary>
    public class History
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="History"/> class.
        /// </summary>
        /// <param name="report">The report.</param>
        internal History(IReport report)
        {
            this.Reports = new List<Report>();
            report.Updated += ((x, y) =>
            {
                Report r = x as Report;
                if (r == null) return;
                Report cloned = (Report)report.Clone();
                this.Reports.Add(cloned);
            });
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the list of historised reports.
        /// </summary>
        /// <value>The reports.</value>
        public List<Report> Reports
        {
            get;
            private set;
        }

        #endregion Properties
    }

    /// <summary>
    /// This report stores information about execution 
    /// time of an instance of a Benchmark.
    /// </summary>
    public class Report : ICloneable, IReport
    {
        #region Fields

        private TimeSpan elapsed;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Report"/> class.
        /// </summary>
        internal Report()
        {
            this.Total = new TimeSpan();
            this.Start = DateTime.Now;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when this report is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the name of the calling method.
        /// </summary>
        /// <value>The calling method.</value>
        public string CallingMethod
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the elapsed time for the last execution.
        /// </summary>
        /// <value>The elapsed timed.</value>
        public TimeSpan Elapsed
        {
            get
            {
                return this.elapsed;
            }
            set
            {
                this.elapsed = value;
                this.Total += value;
                if (this.Updated != null)
                    this.Updated(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Gets the elapsed time in second for the last
        /// execution.
        /// </summary>
        /// <value>The elapsed time in seconds.</value>
        public double ElapsedSeconds
        {
            get
            {
                return this.Elapsed.TotalSeconds;
            }
        }

        /// <summary>
        /// Gets or sets the start date and time.
        /// </summary>
        /// <value>The start.</value>
        public DateTime Start
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the start date and time as a string.
        /// </summary>
        /// <value>The start as string.</value>
        public string StartAsString
        {
            get
            {
                return string.Format("[{0}] {1:00}:{2:00}:{3:00}:{4:000}"
                , this.Start.ToShortDateString()
                , this.Start.Hour, this.Start.Minute, this.Start.Second, this.Start.Millisecond);
            }
        }

        /// <summary>
        /// Gets or sets the total time of execution.
        /// </summary>
        /// <value>The total.</value>
        public TimeSpan Total
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the total time of execution in seconds.
        /// </summary>
        /// <value>The total seconds.</value>
        public double TotalSeconds
        {
            get
            {
                return this.Total.TotalSeconds;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public object Clone()
        {
            Report cloned = new Report();
            cloned.CallingMethod = this.CallingMethod;
            cloned.Elapsed = new TimeSpan(this.Elapsed.Ticks);
            cloned.Total = new TimeSpan(this.Total.Ticks);
            return cloned;
        }

        /// <summary>
        /// Return an object that contains all the
        /// history of the uses of the report specified in the
        /// constructor of this instance.
        /// </summary>
        /// <returns></returns>
        public History Historise()
        {
            return new History(this);
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("{0} [{1:0.000 sec}]", this.CallingMethod, this.Elapsed);
        }

        #endregion Methods
    }
}